@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -
 #
-# This file is part of couchdbkit released under the MIT license. 
+# This file is part of couchdbkit released under the MIT license.
 # See the NOTICE for more information.
 
 """ module that provides a Document object that allows you
@@ -13,7 +13,7 @@ from .properties import value_to_python, \
 convert_property, MAP_TYPES_PROPERTIES, ALLOWED_PROPERTY_TYPES, \
 LazyDict, LazyList
 from ..exceptions import DuplicatePropertyError, ResourceNotFound, \
-ReservedWordError 
+ReservedWordError
 
 
 __all__ = ['ReservedWordError', 'ALLOWED_PROPERTY_TYPES', 'DocumentSchema',
@@ -24,6 +24,7 @@ _RESERVED_WORDS = ['_id', '_rev', '$schema']
 
 _NODOC_WORDS = ['doc_type']
 
+
 def check_reserved_words(attr_name):
     if attr_name in _RESERVED_WORDS:
         raise ReservedWordError(
@@ -90,6 +91,7 @@ class DocumentSchema(object):
     _allow_dynamic_properties = True
     _doc = None
     _db = None
+    _doc_type_attr = 'doc_type'
 
     def __init__(self, _d=None, **properties):
         self._dynamic_properties = {}
@@ -101,8 +103,8 @@ class DocumentSchema(object):
             properties.update(_d)
 
         doc_type = getattr(self, '_doc_type', self.__class__.__name__)
-        self._doc['doc_type'] = doc_type
-        
+        self._doc[self._doc_type_attr] = doc_type
+
         for prop in self._properties.values():
             if prop.name in properties:
                 value = properties.pop(prop.name)
@@ -150,9 +152,9 @@ class DocumentSchema(object):
         return all_properties
 
     def to_json(self):
-        if self._doc.get('doc_type') is None:
+        if self._doc.get(self._doc_type_attr) is None:
             doc_type = getattr(self, '_doc_type', self.__class__.__name__)
-            self._doc['doc_type'] = doc_type
+            self._doc[self._doc_type_attr] = doc_type
         return self._doc
 
     #TODO: add a way to maintain custom dynamic properties
@@ -328,7 +330,7 @@ class DocumentSchema(object):
                     continue
                 elif attr_name.startswith('_'):
                     continue
-                elif attr_name == 'doc_type':
+                elif attr_name == cls._doc_type_attr:
                     continue
                 else:
                     value = value_to_python(value)
@@ -398,14 +400,14 @@ class DocumentBase(DocumentSchema):
 
     def __init__(self, _d=None, **kwargs):
         _d = _d or {}
-        
+
         docid = kwargs.pop('_id', _d.pop("_id", ""))
         docrev = kwargs.pop('_rev', _d.pop("_rev", ""))
-        
+
         DocumentSchema.__init__(self, _d, **kwargs)
-        
+
         if docid: self._doc['_id'] = valid_id(docid)
-        if docrev: self._doc['_rev'] = docrev            
+        if docrev: self._doc['_rev'] = docrev
 
     @classmethod
     def set_db(cls, db):
@@ -461,7 +463,7 @@ class DocumentBase(DocumentSchema):
     def get(cls, docid, rev=None, db=None, dynamic_properties=True):
         """ get document with `docid`
         """
-        if not db:
+        if db is None:
             db = cls.get_db()
         cls._allow_dynamic_properties = dynamic_properties
         return db.get(docid, rev=rev, wrapper=cls.wrap)
@@ -469,8 +471,8 @@ class DocumentBase(DocumentSchema):
     @classmethod
     def get_or_create(cls, docid=None, db=None, dynamic_properties=True, **params):
         """ get  or create document with `docid` """
-       
-        if db:
+
+        if db is not None:
             cls.set_db(db)
         cls._allow_dynamic_properties = dynamic_properties
         db = cls.get_db()
@@ -498,9 +500,9 @@ class DocumentBase(DocumentSchema):
         """
         if self.new_document:
             raise TypeError("the document is not saved")
-        
+
         db = self.get_db()
-        
+
         # delete doc
         db.delete_doc(self._id)
 
@@ -526,7 +528,7 @@ class AttachmentMixin(object):
 
         @return: bool, True if everything was ok.
         """
-        db = self.get_db() 
+        db = self.get_db()
         return db.put_attachment(self._doc, content, name=name,
             content_type=content_type, content_length=content_length)
 
@@ -563,57 +565,7 @@ class QueryMixin(object):
     """ Mixin that add query methods """
 
     @classmethod
-    def __view(cl, view_type=None, data=None, wrapper=None,
-    dynamic_properties=True, wrap_doc=True, classes=None, **params):
-        """
-        The default wrapper can distinguish between multiple Document
-        classes and wrap the result accordingly. The known classes are
-        passed either as classes={<doc_type>: <Document-class>, ...} or
-        classes=[<Document-class1>, <Document-class2>, ...]
-        """
-
-        def default_wrapper(row):
-            data = row.get('value')
-            docid = row.get('id')
-            doc = row.get('doc')
-            if doc is not None and wrap_doc:
-                cls = classes.get(doc.get('doc_type')) if classes else cl
-                cls._allow_dynamic_properties = dynamic_properties
-                return cls.wrap(doc)
-
-            elif not data or data is None:
-                return row
-            elif not isinstance(data, dict) or not docid:
-                return row
-            else:
-                cls = classes.get(data.get('doc_type')) if classes else cl
-                data['_id'] = docid
-                if 'rev' in data:
-                    data['_rev'] = data.pop('rev')
-                cls._allow_dynamic_properties = dynamic_properties
-                return cls.wrap(data)
-
-        if isinstance(classes, list):
-            classes = dict([(c._doc_type, c) for c in classes])
-
-        if wrapper is None:
-            wrapper = default_wrapper
-
-        if not wrapper:
-            wrapper = None
-        elif not callable(wrapper):
-            raise TypeError("wrapper is not a callable")
-
-        db = cl.get_db()
-        if view_type == 'view':
-            return db.view(data, wrapper=wrapper, **params)
-        elif view_type == 'temp_view':
-            return db.temp_view(data, wrapper=wrapper, **params)
-        else:
-            raise RuntimeError("bad view_type : %s" % view_type )
-
-    @classmethod
-    def view(cls, view_name, wrapper=None, dynamic_properties=True,
+    def view(cls, view_name, wrapper=None, dynamic_properties=None,
     wrap_doc=True, classes=None, **params):
         """ Get documents associated view a view.
         Results of view are automatically wrapped
@@ -630,12 +582,13 @@ class QueryMixin(object):
         @return: :class:`simplecouchdb.core.ViewResults` instance. All
         results are wrapped to current document instance.
         """
-        return cls.__view(view_type="view", data=view_name, wrapper=wrapper,
+        db = cls.get_db()
+        return db.view(view_name,
             dynamic_properties=dynamic_properties, wrap_doc=wrap_doc,
-            classes=classes, **params)
+            wrapper=wrapper, schema=classes or cls, **params)
 
     @classmethod
-    def temp_view(cls, design, wrapper=None, dynamic_properties=True,
+    def temp_view(cls, design, wrapper=None, dynamic_properties=None,
     wrap_doc=True, classes=None, **params):
         """ Slow view. Like in view method,
         results are automatically wrapped to
@@ -651,9 +604,10 @@ class QueryMixin(object):
         @return: Like view, return a :class:`simplecouchdb.core.ViewResults`
         instance. All results are wrapped to current document instance.
         """
-        return cls.__view(view_type="temp_view", data=design, wrapper=wrapper,
+        db = cls.get_db()
+        return db.temp_view(design,
             dynamic_properties=dynamic_properties, wrap_doc=wrap_doc,
-            classes=classes, **params)
+            wrapper=wrapper, schema=classes or cls, **params)
 
 class Document(DocumentBase, QueryMixin, AttachmentMixin):
     """
